@@ -137,14 +137,16 @@ impl std::fmt::Display for DisplayDiagnostics<'_> {
             }
             #[cfg(feature = "junit")]
             DiagnosticFormat::Junit => {
-                junit::JunitRenderer::new(self.resolver).render(f, self.diagnostics)?;
+                junit::JunitRenderer::new(self.resolver, self.config.program)
+                    .render(f, self.diagnostics)?;
             }
             #[cfg(feature = "serde")]
             DiagnosticFormat::Gitlab => {
                 gitlab::GitlabRenderer::new(self.resolver).render(f, self.diagnostics)?;
             }
             DiagnosticFormat::Github => {
-                GithubRenderer::new(self.resolver, "ty").render(f, self.diagnostics)?;
+                GithubRenderer::new(self.resolver, self.config.program)
+                    .render(f, self.diagnostics)?;
             }
         }
 
@@ -2538,7 +2540,7 @@ watermelon
         pub(super) fn new() -> TestEnvironment {
             TestEnvironment {
                 db: TestDb::new(),
-                config: DisplayDiagnosticConfig::default(),
+                config: DisplayDiagnosticConfig::new("ty"),
             }
         }
 
@@ -2549,16 +2551,14 @@ watermelon
             // be `Copy` (which it could be, at time of writing, 2025-03-07),
             // but it seems likely to me that it will grow non-`Copy`
             // configuration. So just deal with this inconvenience for now.
-            let mut config = std::mem::take(&mut self.config);
-            config = config.context(lines);
-            self.config = config;
+            let config = self.config.clone();
+            self.config = config.context(lines);
         }
 
         /// Set the output format to use in diagnostic rendering.
         pub(super) fn format(&mut self, format: DiagnosticFormat) {
-            let mut config = std::mem::take(&mut self.config);
-            config = config.format(format);
-            self.config = config;
+            let config = self.config.clone();
+            self.config = config.format(format);
         }
 
         /// Enable preview functionality for diagnostic rendering.
@@ -2567,37 +2567,32 @@ watermelon
             reason = "This is currently only used for JSON but will be needed soon for other formats"
         )]
         pub(super) fn preview(&mut self, yes: bool) {
-            let mut config = std::mem::take(&mut self.config);
-            config = config.preview(yes);
-            self.config = config;
+            let config = self.config.clone();
+            self.config = config.preview(yes);
         }
 
         /// Hide diagnostic severity when rendering.
         pub(super) fn hide_severity(&mut self, yes: bool) {
-            let mut config = std::mem::take(&mut self.config);
-            config = config.hide_severity(yes);
-            self.config = config;
+            let config = self.config.clone();
+            self.config = config.hide_severity(yes);
         }
 
         /// Show fix availability when rendering.
         pub(super) fn show_fix_status(&mut self, yes: bool) {
-            let mut config = std::mem::take(&mut self.config);
-            config = config.with_show_fix_status(yes);
-            self.config = config;
+            let config = self.config.clone();
+            self.config = config.with_show_fix_status(yes);
         }
 
         /// Show a diff for the fix when rendering.
         pub(super) fn show_fix_diff(&mut self, yes: bool) {
-            let mut config = std::mem::take(&mut self.config);
-            config = config.show_fix_diff(yes);
-            self.config = config;
+            let config = self.config.clone();
+            self.config = config.show_fix_diff(yes);
         }
 
         /// The lowest fix applicability to show when rendering.
         pub(super) fn fix_applicability(&mut self, applicability: Applicability) {
-            let mut config = std::mem::take(&mut self.config);
-            config = config.with_fix_applicability(applicability);
-            self.config = config;
+            let config = self.config.clone();
+            self.config = config.with_fix_applicability(applicability);
         }
 
         /// Add a file with the given path and contents to this environment.
@@ -2978,6 +2973,36 @@ if call(foo
         (env, diagnostics)
     }
 
+    /// Create Ruff-style diagnostics with sub-diagnostics for testing the various output formats.
+    pub(crate) fn create_sub_diagnostics(
+        format: DiagnosticFormat,
+    ) -> (TestEnvironment, Vec<Diagnostic>) {
+        let mut env = TestEnvironment::new();
+        env.add("/some/path/def.py", "def f(): pass");
+        env.add("call.py", "f()");
+        env.format(format);
+
+        let mut primary_diagnostic = env
+            .builder("undefined-name", Severity::Error, "Undefined name `f`")
+            .primary("call.py", "1:0", "1:1", "")
+            .secondary_code("F821")
+            .noqa_offset(ruff_text_size::TextSize::from(0))
+            .documentation_url("https://docs.astral.sh/ruff/rules/undefined-name")
+            .build();
+
+        let sub_diagnostic = env
+            .sub_builder(
+                SubDiagnosticSeverity::Info,
+                "Did you mean to import it from `/some/path/def.py`?",
+            )
+            .primary("/some/path/def.py", "1:4", "1:5", "`f` is defined here")
+            .build();
+
+        primary_diagnostic.sub(sub_diagnostic);
+
+        (env, vec![primary_diagnostic])
+    }
+
     /// A Jupyter notebook for testing diagnostics.
     ///
     ///
