@@ -18,6 +18,9 @@ use crate::{checkers::ast::Checker, fix::edits::add_argument};
 /// It's recommended to use a `stacklevel` of 2 or higher, to give the caller
 /// more context about the warning.
 ///
+/// In Python 3.12 and higher, one may also use `skip_file_prefixes` to specify
+/// which file prefixes are ignored when counting the stack level. This implicitly overrides the `stacklevel` to be
+/// at least 2, according to the [Python documentation].
 /// ## Example
 /// ```python
 /// import warnings
@@ -39,7 +42,9 @@ use crate::{checkers::ast::Checker, fix::edits::add_argument};
 /// higher stacklevel to address the diagnostic.
 ///
 /// ## References
-/// - [Python documentation: `warnings.warn`](https://docs.python.org/3/library/warnings.html#warnings.warn)
+/// - [Python documentation: `warnings.warn`][Python documentation]
+///
+///[Python documentation]: https://docs.python.org/3/library/warnings.html#warnings.warn
 #[derive(ViolationMetadata)]
 #[violation_metadata(stable_since = "v0.0.257")]
 pub(crate) struct NoExplicitStacklevel;
@@ -92,12 +97,7 @@ pub(crate) fn no_explicit_stacklevel(checker: &Checker, call: &ast::ExprCall) {
     }
     let mut diagnostic = checker.report_diagnostic(NoExplicitStacklevel, call.func.range());
 
-    let edit = add_argument(
-        "stacklevel=2",
-        &call.arguments,
-        checker.comment_ranges(),
-        checker.locator().contents(),
-    );
+    let edit = add_argument("stacklevel=2", &call.arguments, checker.tokens());
 
     diagnostic.set_fix(Fix::unsafe_edit(edit));
 }
