@@ -1,5 +1,6 @@
 use std::io::StdoutLock;
 
+use anyhow::Result;
 use indicatif::ProgressDrawTarget;
 
 use crate::logging::VerbosityLevel;
@@ -11,19 +12,10 @@ pub(crate) struct Printer {
 }
 
 impl Printer {
-    #[must_use]
-    pub(crate) fn with_no_progress(self) -> Self {
-        Self {
-            verbosity: self.verbosity,
-            no_progress: true,
-        }
-    }
-
-    #[must_use]
-    pub(crate) fn with_verbosity(self, verbosity: VerbosityLevel) -> Self {
+    pub(crate) fn new(verbosity: VerbosityLevel, no_progress: bool) -> Self {
         Self {
             verbosity,
-            no_progress: self.no_progress,
+            no_progress,
         }
     }
 
@@ -109,6 +101,11 @@ impl Printer {
     pub(crate) fn stream_for_details(self) -> Stdout {
         self.stdout_general()
     }
+
+    pub(crate) fn clear_screen() -> Result<()> {
+        clearscreen::clear()?;
+        Ok(())
+    }
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -162,6 +159,22 @@ impl Stdout {
     }
 }
 
+impl std::io::Write for Stdout {
+    fn write(&mut self, buf: &[u8]) -> std::io::Result<usize> {
+        match self.status {
+            StreamStatus::Enabled => self.handle().write(buf),
+            StreamStatus::Disabled => Ok(buf.len()),
+        }
+    }
+
+    fn flush(&mut self) -> std::io::Result<()> {
+        match self.status {
+            StreamStatus::Enabled => self.handle().flush(),
+            StreamStatus::Disabled => Ok(()),
+        }
+    }
+}
+
 impl std::fmt::Write for Stdout {
     fn write_str(&mut self, s: &str) -> std::fmt::Result {
         match self.status {
