@@ -518,7 +518,7 @@ impl CompletionAnswer {
     fn qualified(&self) -> String {
         self.module
             .as_ref()
-            .map(|module| format!("{module}.{}", self.symbol))
+            .map(|module| format!("{module}{}", self.symbol))
             .unwrap_or_else(|| self.symbol.clone())
     }
 }
@@ -594,7 +594,7 @@ fn copy_file(src: &SystemPath, dst: &SystemPath) -> anyhow::Result<Vec<Cursor>>
         // Our module/symbol identifier regex here is certainly more
         // permissive than necessary, but I think that should be fine
         // for this silly little syntax. ---AG
-        Regex::new(r"<CURSOR:\s*(?:(?<module>[\S--.]+)\.)?(?<symbol>[\S--.]+)>").unwrap()
+        Regex::new(r"<CURSOR:\s*(?<module>(?:[\S--.]+\.)*)(?<symbol>[\S--.]+)>").unwrap()
     });
 
     let src_data =
@@ -619,14 +619,18 @@ fn copy_file(src: &SystemPath, dst: &SystemPath) -> anyhow::Result<Vec<Cursor>>
         let symbol = str::from_utf8(&caps["symbol"])
             .context("expected symbol in cursor directive in `{src}` is not valid UTF-8")?
             .to_string();
-        let module = caps
-            .name("module")
-            .map(|module| {
-                str::from_utf8(module.as_bytes())
-                    .context("expected module in cursor directive in `{src}` is not valid UTF-8")
-            })
-            .transpose()?
-            .map(ToString::to_string);
+        let module =
+            caps.name("module")
+                .and_then(|module| {
+                    if module.as_bytes().is_empty() {
+                        return None;
+                    }
+                    Some(str::from_utf8(module.as_bytes()).context(
+                        "expected module in cursor directive in `{src}` is not valid UTF-8",
+                    ))
+                })
+                .transpose()?
+                .map(ToString::to_string);
         let answer = CompletionAnswer { symbol, module };
         cursors.push(Cursor {
             path: dst.to_path_buf(),
