@@ -683,6 +683,12 @@ pub struct EnvironmentOptions {
     /// ty uses the `site-packages` directory of your project's Python environment
     /// to resolve third-party (and, in some cases, first-party) imports in your code.
     ///
+    /// This can be a path to:
+    ///
+    /// - A Python interpreter, e.g. `.venv/bin/python3`
+    /// - A virtual environment directory, e.g. `.venv`
+    /// - A system Python [`sys.prefix`] directory, e.g. `/usr`
+    ///
     /// If you're using a project management tool such as uv, you should not generally need to
     /// specify this option, as commands such as `uv run` will set the `VIRTUAL_ENV` environment
     /// variable to point to your project's virtual environment. ty can also infer the location of
@@ -690,10 +696,7 @@ pub struct EnvironmentOptions {
     /// in the project root if none of the above apply. Failing that, ty will look for a `python3`
     /// or `python` binary available in `PATH`.
     ///
-    /// Passing a path to a Python executable is supported, but passing a path to a dynamic executable
-    /// (such as a shim) is not currently supported.
-    ///
-    /// This option can be used to point to virtual or system Python environments.
+    /// [`sys.prefix`]: https://docs.python.org/3/library/sys.html#sys.prefix
     #[serde(skip_serializing_if = "Option::is_none")]
     #[option(
         default = r#"null"#,
@@ -1326,6 +1329,31 @@ pub struct AnalysisOptions {
         "#
     )]
     pub allowed_unresolved_imports: Option<Vec<RangedValue<String>>>,
+
+    /// A list of module glob patterns whose imports should be replaced with `typing.Any`.
+    ///
+    /// Unlike `allowed-unresolved-imports`, this setting replaces the module's type information
+    /// with `typing.Any` even if the module can be resolved. Import diagnostics are
+    /// unconditionally suppressed for matching modules.
+    ///
+    /// - Prefix a pattern with `!` to exclude matching modules
+    ///
+    /// When multiple patterns match, later entries take precedence.
+    ///
+    /// Glob patterns can be used in combinations with each other. For example, to suppress errors for
+    /// any module where the first component contains the substring `test`, use `*test*.**`.
+    ///
+    /// When multiple patterns match, later entries take precedence.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    #[option(
+        default = r#"[]"#,
+        value_type = "list[str]",
+        example = r#"
+            # Replace all pandas and numpy imports with Any
+            replace-imports-with-any = ["pandas.**", "numpy.**"]
+        "#
+    )]
+    pub replace_imports_with_any: Option<Vec<RangedValue<String>>>,
 }
 
 impl AnalysisOptions {
@@ -1337,11 +1365,13 @@ impl AnalysisOptions {
         let Self {
             respect_type_ignore_comments,
             allowed_unresolved_imports,
+            replace_imports_with_any,
         } = self;
 
         let AnalysisSettings {
             respect_type_ignore_comments: respect_type_ignore_default,
             allowed_unresolved_imports: allowed_unresolved_imports_default,
+            replace_imports_with_any: replace_imports_with_any_default,
         } = AnalysisSettings::default();
 
         let allowed_unresolved_imports =
@@ -1355,10 +1385,22 @@ impl AnalysisOptions {
                 allowed_unresolved_imports_default
             };
 
+        let replace_imports_with_any =
+            if let Some(replace_imports_with_any) = replace_imports_with_any {
+                build_module_glob_set(db, replace_imports_with_any, "replace_imports_with_any")
+                    .unwrap_or_else(|error| {
+                        diagnostics.push(*error);
+                        ModuleGlobSet::empty()
+                    })
+            } else {
+                replace_imports_with_any_default
+            };
+
         AnalysisSettings {
             respect_type_ignore_comments: respect_type_ignore_comments
                 .unwrap_or(respect_type_ignore_default),
             allowed_unresolved_imports,
+            replace_imports_with_any,
         }
     }
 }
