@@ -1,10 +1,11 @@
+use crate::diagnostic::did_you_mean;
 use core::fmt;
 use itertools::Itertools;
 use ruff_db::diagnostic::{DiagnosticId, LintName, Severity};
 use rustc_hash::FxHashMap;
+use std::error::Error;
 use std::fmt::Formatter;
 use std::hash::Hasher;
-use thiserror::Error;
 
 #[derive(Debug, Clone)]
 pub struct LintMetadata {
@@ -32,7 +33,7 @@ pub struct LintMetadata {
     pub line: u32,
 }
 
-#[derive(Copy, Clone, Debug, Eq, PartialEq, Ord, PartialOrd, Hash)]
+#[derive(Copy, Clone, Debug, Eq, PartialEq, Ord, PartialOrd, Hash, get_size2::GetSize)]
 #[cfg_attr(
     feature = "serde",
     derive(serde::Serialize, serde::Deserialize),
@@ -115,6 +116,10 @@ impl LintMetadata {
         self.documentation_lines().join("\n")
     }
 
+    pub fn documentation_url(&self) -> String {
+        lint_documentation_url(self.name())
+    }
+
     pub fn default_level(&self) -> Level {
         self.default_level
     }
@@ -132,6 +137,10 @@ impl LintMetadata {
     }
 }
 
+pub fn lint_documentation_url(lint_name: LintName) -> String {
+    format!("https://ty.dev/rules#{lint_name}")
+}
+
 #[doc(hidden)]
 pub const fn lint_metadata_defaults() -> LintMetadata {
     LintMetadata {
@@ -380,7 +389,12 @@ impl LintRegistry {
                     }
                 }
 
-                Err(GetLintError::Unknown(code.to_string()))
+                let suggestion = did_you_mean(self.by_name.keys().copied(), code);
+
+                Err(GetLintError::Unknown {
+                    code: code.to_string(),
+                    suggestion: suggestion.map(str::to_string),
+                })
             }
         }
     }
@@ -415,25 +429,45 @@ impl LintRegistry {
     }
 }
 
-#[derive(Error, Debug, Clone, PartialEq, Eq, get_size2::GetSize)]
+#[derive(Debug, Clone, PartialEq, Eq, get_size2::GetSize)]
 pub enum GetLintError {
     /// The name maps to this removed lint.
-    #[error("lint `{0}` has been removed")]
     Removed(LintName),
 
     /// No lint with the given name is known.
-    #[error("unknown lint `{0}`")]
-    Unknown(String),
+    Unknown {
+        code: String,
+        suggestion: Option<String>,
+    },
 
     /// The name uses the full qualified diagnostic id `lint:<rule>` instead of just `rule`.
     /// The String is the name without the `lint:` category prefix.
-    #[error("unknown lint `{prefixed}`. Did you mean `{suggestion}`?")]
     PrefixedWithCategory {
         prefixed: String,
         suggestion: String,
     },
 }
 
+impl Error for GetLintError {}
+
+impl std::fmt::Display for GetLintError {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        match self {
+            GetLintError::Removed(code) => write!(f, "Removed rule `{code}`"),
+            GetLintError::Unknown { code, suggestion } => match suggestion {
+                None => write!(f, "Unknown rule `{code}`"),
+                Some(suggestion) => {
+                    write!(f, "Unknown rule `{code}`. Did you mean `{suggestion}`?")
+                }
+            },
+            GetLintError::PrefixedWithCategory {
+                prefixed,
+                suggestion,
+            } => write!(f, "Unknown rule `{prefixed}`. Did you mean `{suggestion}`?"),
+        }
+    }
+}
+
 #[derive(Debug, PartialEq, Eq, Clone, Copy)]
 pub enum LintEntry {
     /// An existing lint rule. Can be in preview, stable or deprecated.
@@ -463,7 +497,7 @@ impl From<&'static LintMetadata> for LintEntry {
     }
 }
 
-#[derive(Debug, Clone, Default, PartialEq, Eq, get_size2::GetSize)]
+#[derive(Clone, Default, PartialEq, Eq, get_size2::GetSize)]
 pub struct RuleSelection {
     /// Map with the severity for each enabled lint rule.
     ///
@@ -541,6 +575,35 @@ impl RuleSelection {
     }
 }
 
+// The default `LintId` debug implementation prints the entire lint metadata.
+// This is way too verbose.
+impl fmt::Debug for RuleSelection {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        let lints = self.lints.iter().sorted_by_key(|(lint, _)| lint.name);
+
+        if f.alternate() {
+            let mut f = f.debug_map();
+
+            for (lint, (severity, source)) in lints {
+                f.entry(
+                    &lint.name().as_str(),
+                    &format_args!("{severity:?} ({source:?})"),
+                );
+            }
+
+            f.finish()
+        } else {
+            let mut f = f.debug_set();
+
+            for (lint, _) in lints {
+                f.entry(&lint.name());
+            }
+
+            f.finish()
+        }
+    }
+}
+
 #[derive(Default, Copy, Clone, Debug, PartialEq, Eq, get_size2::GetSize)]
 pub enum LintSource {
     /// The user didn't enable the rule explicitly, instead it's enabled by default.
@@ -552,4 +615,7 @@ pub enum LintSource {
 
     /// The rule was enabled in a configuration file.
     File,
+
+    /// The rule was enabled from the configuration in the editor.
+    Editor,
 }
