@@ -33,11 +33,6 @@ impl ScopedPredicateId {
     fn is_terminal(self) -> bool {
         self >= Self::SMALLEST_TERMINAL
     }
-
-    #[cfg(test)]
-    pub(crate) fn as_u32(self) -> u32 {
-        self.0
-    }
 }
 
 impl Idx for ScopedPredicateId {
@@ -117,7 +112,7 @@ pub(crate) enum PredicateNode<'db> {
     StarImportPlaceholder(StarImportPlaceholderPredicate<'db>),
 }
 
-#[derive(Debug, Clone, Copy, Hash, PartialEq, Eq, salsa::Update)]
+#[derive(Debug, Clone, Copy, Hash, PartialEq, Eq, salsa::Update, get_size2::GetSize)]
 pub(crate) enum ClassPatternKind {
     Irrefutable,
     Refutable,
@@ -130,7 +125,7 @@ impl ClassPatternKind {
 }
 
 /// Pattern kinds for which we support type narrowing and/or static reachability analysis.
-#[derive(Debug, Clone, Hash, PartialEq, salsa::Update)]
+#[derive(Debug, Clone, Hash, PartialEq, salsa::Update, get_size2::GetSize)]
 pub(crate) enum PatternPredicateKind<'db> {
     Singleton(Singleton),
     Value(Expression<'db>),
@@ -140,7 +135,7 @@ pub(crate) enum PatternPredicateKind<'db> {
     Unsupported,
 }
 
-#[salsa::tracked(debug)]
+#[salsa::tracked(debug, heap_size=ruff_memory_usage::heap_size)]
 pub(crate) struct PatternPredicate<'db> {
     pub(crate) file: File,
 
@@ -166,10 +161,10 @@ impl<'db> PatternPredicate<'db> {
     }
 }
 
-/// A "placeholder predicate" that is used to model the fact that the boundness of a
-/// (possible) definition or declaration caused by a `*` import cannot be fully determined
-/// until type-inference time. This is essentially the same as a standard reachability constraint,
-/// so we reuse the [`Predicate`] infrastructure to model it.
+/// A "placeholder predicate" that is used to model the fact that the boundness of a (possible)
+/// definition or declaration caused by a `*` import cannot be fully determined until type-
+/// inference time. This is essentially the same as a standard reachability constraint, so we reuse
+/// the [`Predicate`] infrastructure to model it.
 ///
 /// To illustrate, say we have a module `exporter.py` like so:
 ///
@@ -183,14 +178,14 @@ impl<'db> PatternPredicate<'db> {
 /// ```py
 /// A = 1
 ///
-/// from importer import *
+/// from exporter import *
 /// ```
 ///
-/// Since we cannot know whether or not <condition> is true at semantic-index time,
-/// we record a definition for `A` in `b.py` as a result of the `from a import *`
-/// statement, but place a predicate on it to record the fact that we don't yet
-/// know whether this definition will be visible from all control-flow paths or not.
-/// Essentially, we model `b.py` as something similar to this:
+/// Since we cannot know whether or not <condition> is true at semantic-index time, we record
+/// a definition for `A` in `importer.py` as a result of the `from exporter import *` statement,
+/// but place a predicate on it to record the fact that we don't yet know whether this definition
+/// will be visible from all control-flow paths or not. Essentially, we model `importer.py` as
+/// something similar to this:
 ///
 /// ```py
 /// A = 1
@@ -199,14 +194,14 @@ impl<'db> PatternPredicate<'db> {
 ///     from a import A
 /// ```
 ///
-/// At type-check time, the placeholder predicate for the `A` definition is evaluated by
-/// attempting to resolve the `A` symbol in `a.py`'s global namespace:
+/// At type-check time, the placeholder predicate for the `A` definition is evaluated by attempting
+/// to resolve the `A` symbol in `exporter.py`'s global namespace:
 /// - If it resolves to a definitely bound symbol, then the predicate resolves to [`Truthiness::AlwaysTrue`]
 /// - If it resolves to an unbound symbol, then the predicate resolves to [`Truthiness::AlwaysFalse`]
 /// - If it resolves to a possibly bound symbol, then the predicate resolves to [`Truthiness::Ambiguous`]
 ///
 /// [Truthiness]: [crate::types::Truthiness]
-#[salsa::tracked(debug)]
+#[salsa::tracked(debug, heap_size=ruff_memory_usage::heap_size)]
 pub(crate) struct StarImportPlaceholderPredicate<'db> {
     pub(crate) importing_file: File,
 
