@@ -5,13 +5,13 @@ use crate::place::builtins_module_scope;
 use crate::semantic_index::definition::Definition;
 use crate::semantic_index::definition::DefinitionKind;
 use crate::semantic_index::{attribute_scopes, global_scope, semantic_index, use_def_map};
-use crate::types::call::{CallArguments, MatchedArgument};
+use crate::types::call::{CallArguments, CallError, MatchedArgument};
 use crate::types::signatures::{ParameterKind, Signature};
 use crate::types::{
     CallDunderError, CallableTypes, ClassBase, ClassLiteral, ClassType, KnownUnion, Type,
     TypeContext, UnionType,
 };
-use crate::{Db, DisplaySettings, HasType, SemanticModel};
+use crate::{Db, DisplaySettings, HasDefinition, HasType, SemanticModel};
 use itertools::Either;
 use ruff_db::files::FileRange;
 use ruff_db::parsed::parsed_module;
@@ -95,13 +95,17 @@ pub fn definitions_for_name<'db>(
 
                 for binding in global_bindings {
                     if let Some(def) = binding.binding.definition() {
-                        all_definitions.insert(def);
+                        if def.kind(db).is_user_visible() {
+                            all_definitions.insert(def);
+                        }
                     }
                 }
 
                 for declaration in global_declarations {
                     if let Some(def) = declaration.declaration.definition() {
-                        all_definitions.insert(def);
+                        if def.kind(db).is_user_visible() {
+                            all_definitions.insert(def);
+                        }
                     }
                 }
             }
@@ -122,13 +126,17 @@ pub fn definitions_for_name<'db>(
 
         for binding in bindings {
             if let Some(def) = binding.binding.definition() {
-                all_definitions.insert(def);
+                if def.kind(db).is_user_visible() {
+                    all_definitions.insert(def);
+                }
             }
         }
 
         for declaration in declarations {
             if let Some(def) = declaration.declaration.definition() {
-                all_definitions.insert(def);
+                if def.kind(db).is_user_visible() {
+                    all_definitions.insert(def);
+                }
             }
         }
 
@@ -404,6 +412,49 @@ fn definitions_for_attribute_in_class_hierarchy<'db>(
     resolved
 }
 
+pub struct TypedDictKeyHover<'db> {
+    pub owner: String,
+    pub key: String,
+    pub declared_ty: Type<'db>,
+    pub docstring: Option<String>,
+}
+
+pub fn typed_dict_key_definition<'db>(
+    model: &SemanticModel<'db>,
+    subscript: &ast::ExprSubscript,
+    key: &str,
+) -> Option<ResolvedDefinition<'db>> {
+    let value_ty = subscript.value.inferred_type(model)?;
+    let typed_dict = value_ty.as_typed_dict()?;
+    let field = typed_dict.items(model.db()).get(key)?;
+    let definition = field.first_declaration()?;
+    Some(ResolvedDefinition::Definition(definition))
+}
+
+pub fn typed_dict_key_hover<'db>(
+    model: &SemanticModel<'db>,
+    subscript: &ast::ExprSubscript,
+) -> Option<TypedDictKeyHover<'db>> {
+    let key = subscript
+        .slice
+        .as_string_literal_expr()
+        .map(|literal| literal.value.to_str())?;
+    let value_ty = subscript.value.inferred_type(model)?;
+    let typed_dict = value_ty.as_typed_dict()?;
+    let owner = value_ty.display(model.db()).to_string();
+    let field = typed_dict.items(model.db()).get(key)?;
+    let docstring = field
+        .first_declaration()
+        .and_then(|declaration| declaration.docstring(model.db()));
+
+    Some(TypedDictKeyHover {
+        owner,
+        key: key.to_string(),
+        declared_ty: field.declared_ty,
+        docstring,
+    })
+}
+
 /// Returns definitions for a keyword argument in a call expression.
 /// This resolves the keyword argument to the corresponding parameter(s) in the callable's signature(s).
 pub fn definitions_for_keyword_argument<'db>(
@@ -481,6 +532,28 @@ pub fn definitions_for_imported_symbol<'db>(
     )
 }
 
+/// Returns the definition and overload co-definitions for a function declaration.
+///
+/// For overloaded functions this includes sibling overload declarations and the
+/// implementation, if present.
+pub fn definitions_and_overloads_for_function<'db>(
+    model: &SemanticModel<'db>,
+    function: &ast::StmtFunctionDef,
+) -> Vec<ResolvedDefinition<'db>> {
+    if let Some(function_type) = function
+        .inferred_type(model)
+        .and_then(Type::as_function_literal)
+    {
+        function_type
+            .iter_overloads_and_implementation(model.db())
+            .filter_map(|overload| overload.signature(model.db()).definition())
+            .map(ResolvedDefinition::Definition)
+            .collect()
+    } else {
+        vec![ResolvedDefinition::Definition(function.definition(model))]
+    }
+}
+
 /// Details about a callable signature for IDE support.
 #[derive(Debug, Clone)]
 pub struct CallSignatureDetails<'db> {
@@ -514,7 +587,39 @@ pub struct CallSignatureDetails<'db> {
     pub argument_to_parameter_mapping: Vec<MatchedArgument<'db>>,
 }
 
-impl CallSignatureDetails<'_> {
+impl<'db> CallSignatureDetails<'db> {
+    fn from_binding(db: &'db dyn Db, binding: &crate::types::call::Binding<'db>) -> Self {
+        let argument_to_parameter_mapping = binding.argument_matches().to_vec();
+        let specialization = binding.specialization();
+        let signature = binding.signature.clone();
+        let display_details = signature.display(db).to_string_parts();
+        let (parameter_kinds, parameter_types): (Vec<ParameterKind>, Vec<Type>) = signature
+            .parameters()
+            .iter()
+            .map(|param| {
+                // Apply the inferred specialization (if any) to resolve TypeVars
+                // in the annotated type. For example, if `_KT` was inferred as
+                // `str` from the call arguments, this turns `_KT` into `str`.
+                let mut ty = param.annotated_type();
+                if let Some(spec) = specialization {
+                    ty = ty.apply_specialization(db, spec);
+                }
+                (param.kind().clone(), ty)
+            })
+            .unzip();
+
+        CallSignatureDetails {
+            definition: signature.definition(),
+            signature,
+            label: display_details.label,
+            parameter_label_offsets: display_details.parameter_ranges,
+            parameter_names: display_details.parameter_names,
+            parameter_kinds,
+            parameter_types,
+            argument_to_parameter_mapping,
+        }
+    }
+
     fn get_definition_parameter_range(&self, db: &dyn Db, name: &str) -> Option<FileRange> {
         let definition = self.signature.definition()?;
         let file = definition.file(db);
@@ -542,48 +647,36 @@ pub fn call_signature_details<'db>(
         return Vec::new();
     };
 
+    let db = model.db();
+
     // Use into_callable to handle all the complex type conversions
     if let Some(callable_type) = func_type
-        .try_upcast_to_callable(model.db())
-        .map(|callables| callables.into_type(model.db()))
+        .try_upcast_to_callable(db)
+        .map(|callables| callables.into_type(db))
     {
+        // Use from_arguments_typed so that check_types can infer TypeVar
+        // specializations from the actual argument types at this call site.
         let call_arguments =
-            CallArguments::from_arguments(&call_expr.arguments, |_, splatted_value| {
+            CallArguments::from_arguments_typed(&call_expr.arguments, |splatted_value| {
                 splatted_value
                     .inferred_type(model)
                     .unwrap_or(Type::unknown())
             });
-        let bindings = callable_type
-            .bindings(model.db())
-            .match_parameters(model.db(), &call_arguments);
+        let mut bindings = callable_type
+            .bindings(db)
+            .match_parameters(db, &call_arguments);
+
+        // Run type checking to resolve TypeVar bindings from argument types.
+        // For example, calling `dict[str, int].get("a")` resolves the `_KT`
+        // TypeVar to `str`. We ignore errors since we still want signature
+        // details even if the call has type errors.
+        let _ = bindings.check_types_impl(db, &call_arguments, TypeContext::default(), &[]);
 
         // Extract signature details from all callable bindings
         bindings
-            .into_iter()
+            .iter_flat()
             .flatten()
-            .map(|binding| {
-                let argument_to_parameter_mapping = binding.argument_matches().to_vec();
-                let signature = binding.signature;
-                let display_details = signature.display(model.db()).to_string_parts();
-                let parameter_label_offsets = display_details.parameter_ranges;
-                let parameter_names = display_details.parameter_names;
-                let (parameter_kinds, parameter_types): (Vec<ParameterKind>, Vec<Type>) = signature
-                    .parameters()
-                    .iter()
-                    .map(|param| (param.kind().clone(), param.annotated_type()))
-                    .unzip();
-
-                CallSignatureDetails {
-                    definition: signature.definition(),
-                    signature,
-                    label: display_details.label,
-                    parameter_label_offsets,
-                    parameter_names,
-                    parameter_kinds,
-                    parameter_types,
-                    argument_to_parameter_mapping,
-                }
-            })
+            .map(|binding| CallSignatureDetails::from_binding(db, binding))
             .collect()
     } else {
         // Type is not callable, return empty signatures
@@ -621,7 +714,7 @@ pub fn call_type_simplified_by_overloads(
     }
 
     // Hand the overload resolution system as much type info as we have
-    let args = CallArguments::from_arguments_typed(&call_expr.arguments, |_, splatted_value| {
+    let args = CallArguments::from_arguments_typed(&call_expr.arguments, |splatted_value| {
         splatted_value
             .inferred_type(model)
             .unwrap_or(Type::unknown())
@@ -633,7 +726,7 @@ pub fn call_type_simplified_by_overloads(
         .check_types(db, &args, TypeContext::default(), &[])
         // Only use the Ok
         .iter()
-        .flatten()
+        .flat_map(super::call::bind::Bindings::iter_flat)
         .flat_map(|binding| {
             binding.matching_overloads().map(|(_, overload)| {
                 overload
@@ -667,7 +760,7 @@ pub fn definitions_for_bin_op<'db>(
     let callable_type = promote_literals_for_self(model.db(), bindings.callable_type());
 
     let definitions: Vec<_> = bindings
-        .into_iter()
+        .iter_flat()
         .flatten()
         .filter_map(|binding| {
             Some(ResolvedDefinition::Definition(
@@ -725,7 +818,7 @@ pub fn definitions_for_unary_op<'db>(
     let callable_type = promote_literals_for_self(model.db(), bindings.callable_type());
 
     let definitions = bindings
-        .into_iter()
+        .iter_flat()
         .flatten()
         .filter_map(|binding| {
             Some(ResolvedDefinition::Definition(
@@ -796,6 +889,60 @@ pub fn find_active_signature_from_details(
     Some(best_index)
 }
 
+/// Resolve a call expression to its matching overload's signature details,
+/// using full type checking (not just arity matching) for overload resolution.
+///
+/// Falls back to arity-based matching if type-based resolution fails.
+fn resolve_call_signature<'db>(
+    model: &SemanticModel<'db>,
+    call_expr: &ast::ExprCall,
+) -> Option<CallSignatureDetails<'db>> {
+    let db = model.db();
+    let func_type = call_expr.func.inferred_type(model)?;
+    let callable_type = func_type.try_upcast_to_callable(db)?.into_type(db);
+
+    let args = CallArguments::from_arguments_typed(&call_expr.arguments, |splatted_value| {
+        splatted_value
+            .inferred_type(model)
+            .unwrap_or(Type::unknown())
+    });
+
+    // Extract the `Bindings` regardless of whether type checking succeeded or failed.
+    let bindings = callable_type
+        .bindings(db)
+        .match_parameters(db, &args)
+        .check_types(db, &args, TypeContext::default(), &[])
+        .unwrap_or_else(|CallError(_, bindings)| *bindings);
+
+    // First, try to find the matching overload after full type checking.
+    let type_checked_details: Vec<_> = bindings
+        .iter_flat()
+        .flat_map(|binding| binding.matching_overloads().map(|(_, overload)| overload))
+        .map(|binding| CallSignatureDetails::from_binding(db, binding))
+        .collect();
+
+    if !type_checked_details.is_empty() {
+        let active = find_active_signature_from_details(&type_checked_details)?;
+        return type_checked_details.into_iter().nth(active);
+    }
+
+    // If all overloads have type-checking errors (e.g., `InvalidArgumentType`),
+    // `matching_overloads()` returns empty. Fall back to arity-based matching
+    // across all overloads to pick the best candidate for showing hints.
+    let all_details: Vec<_> = bindings
+        .iter_flat()
+        .flatten()
+        .map(|binding| CallSignatureDetails::from_binding(db, binding))
+        .collect();
+
+    if all_details.is_empty() {
+        return None;
+    }
+
+    let active = find_active_signature_from_details(&all_details)?;
+    all_details.into_iter().nth(active)
+}
+
 #[derive(Default)]
 pub struct InlayHintCallArgumentDetails {
     /// The position of the arguments mapped to their name and the range of the argument definition in the signature.
@@ -807,25 +954,14 @@ pub fn inlay_hint_call_argument_details<'db>(
     model: &SemanticModel<'db>,
     call_expr: &ast::ExprCall,
 ) -> Option<InlayHintCallArgumentDetails> {
-    let signature_details = call_signature_details(model, call_expr);
+    let resolved = resolve_call_signature(model, call_expr)?;
 
-    if signature_details.is_empty() {
-        return None;
-    }
-
-    let active_signature_index = find_active_signature_from_details(&signature_details)?;
-
-    let call_signature_details = signature_details.get(active_signature_index)?;
-
-    let parameters = call_signature_details.signature.parameters();
+    let parameters = resolved.signature.parameters();
 
     let mut argument_names = HashMap::new();
 
     for arg_index in 0..call_expr.arguments.args.len() {
-        let Some(arg_mapping) = call_signature_details
-            .argument_to_parameter_mapping
-            .get(arg_index)
-        else {
+        let Some(arg_mapping) = resolved.argument_to_parameter_mapping.get(arg_index) else {
             continue;
         };
 
@@ -847,8 +983,7 @@ pub fn inlay_hint_call_argument_details<'db>(
             continue;
         };
 
-        let parameter_label_offset =
-            call_signature_details.get_definition_parameter_range(db, param.name()?);
+        let parameter_label_offset = resolved.get_definition_parameter_range(db, param.name()?);
 
         // Only add hints for parameters that can be specified by name
         if !param.is_positional_only() && !param.is_variadic() && !param.is_keyword_variadic() {
@@ -920,7 +1055,7 @@ mod resolve_definition {
     }
 
     impl<'db> ResolvedDefinition<'db> {
-        pub(crate) fn definition(&self) -> Option<Definition<'db>> {
+        pub fn definition(&self) -> Option<Definition<'db>> {
             match self {
                 ResolvedDefinition::Definition(definition) => Some(*definition),
                 ResolvedDefinition::Module(_) => None,
@@ -1420,7 +1555,8 @@ mod resolve_definition {
             | DefinitionKind::ExceptHandler(_)
             | DefinitionKind::TypeVar(_)
             | DefinitionKind::ParamSpec(_)
-            | DefinitionKind::TypeVarTuple(_) => {
+            | DefinitionKind::TypeVarTuple(_)
+            | DefinitionKind::LoopHeader(_) => {
                 // Not yet implemented
                 return Err(());
             }
