@@ -1,4 +1,3 @@
-use super::TypeVarVariance;
 use crate::place::PlaceAndQualifiers;
 use crate::semantic_index::definition::Definition;
 use crate::types::constraints::ConstraintSet;
@@ -7,10 +6,10 @@ use crate::types::protocol_class::ProtocolClass;
 use crate::types::relation::{HasRelationToVisitor, IsDisjointVisitor, TypeRelation};
 use crate::types::variance::VarianceInferable;
 use crate::types::{
-    ApplyTypeMappingVisitor, BoundTypeVarInstance, ClassType, DynamicType,
-    FindLegacyTypeVarsVisitor, KnownClass, MaterializationKind, MemberLookupPolicy,
+    ApplyTypeMappingVisitor, BoundTypeVarInstance, ClassLiteral, ClassType, DynamicClassLiteral,
+    DynamicType, FindLegacyTypeVarsVisitor, KnownClass, MaterializationKind, MemberLookupPolicy,
     NormalizedVisitor, SpecialFormType, Type, TypeContext, TypeMapping, TypeVarBoundOrConstraints,
-    TypedDictType, UnionType, todo_type,
+    TypeVarVariance, TypedDictType, UnionType, todo_type,
 };
 use crate::{Db, FxOrderSet};
 
@@ -334,7 +333,7 @@ impl<'db> SubclassOfType<'db> {
     pub(crate) fn is_typed_dict(self, db: &'db dyn Db) -> bool {
         self.subclass_of
             .into_class(db)
-            .is_some_and(|class| class.class_literal(db).0.is_typed_dict(db))
+            .is_some_and(|class| class.class_literal(db).is_typed_dict(db))
     }
 }
 
@@ -393,13 +392,8 @@ impl<'db> SubclassOfInner<'db> {
                         Self::try_from_instance(db, bound)
                             .and_then(|subclass_of| subclass_of.into_class(db))
                     }
-                    Some(TypeVarBoundOrConstraints::Constraints(constraints)) => {
-                        match &**constraints.elements(db) {
-                            [bound] => Self::try_from_instance(db, *bound)
-                                .and_then(|subclass_of| subclass_of.into_class(db)),
-                            _ => Some(ClassType::object(db)),
-                        }
-                    }
+                    // TODO this is quite imprecise
+                    Some(TypeVarBoundOrConstraints::Constraints(_)) => Some(ClassType::object(db)),
                 }
             }
         }
@@ -534,3 +528,9 @@ impl<'db> From<SubclassOfType<'db>> for Type<'db> {
         }
     }
 }
+
+impl<'db> From<DynamicClassLiteral<'db>> for SubclassOfInner<'db> {
+    fn from(value: DynamicClassLiteral<'db>) -> Self {
+        SubclassOfInner::Class(ClassType::NonGeneric(ClassLiteral::Dynamic(value)))
+    }
+}
