@@ -2,10 +2,14 @@ use std::cmp::Ordering;
 
 use salsa::plumbing::AsId;
 
-use crate::{db::Db, types::bound_super::SuperOwnerKind};
+use crate::{
+    db::Db,
+    types::{LiteralValueTypeKind, bound_super::SuperOwnerKind},
+};
 
 use super::{
-    DynamicType, TodoType, Type, TypeIsType, class_base::ClassBase, subclass_of::SubclassOfInner,
+    DynamicType, TodoType, Type, TypeGuardLike, TypeGuardType, TypeIsType, class_base::ClassBase,
+    subclass_of::SubclassOfInner,
 };
 
 /// Return an [`Ordering`] that describes the canonical order in which two types should appear
@@ -46,28 +50,43 @@ pub(super) fn union_or_intersection_elements_ordering<'db>(
         (Type::Never, _) => Ordering::Less,
         (_, Type::Never) => Ordering::Greater,
 
-        (Type::LiteralString, _) => Ordering::Less,
-        (_, Type::LiteralString) => Ordering::Greater,
+        (Type::LiteralValue(left), Type::LiteralValue(right)) => {
+            match (left.kind(), right.kind()) {
+                (LiteralValueTypeKind::LiteralString, _) => Ordering::Less,
+                (_, LiteralValueTypeKind::LiteralString) => Ordering::Greater,
 
-        (Type::BooleanLiteral(left), Type::BooleanLiteral(right)) => left.cmp(right),
-        (Type::BooleanLiteral(_), _) => Ordering::Less,
-        (_, Type::BooleanLiteral(_)) => Ordering::Greater,
+                (LiteralValueTypeKind::Bool(left), LiteralValueTypeKind::Bool(right)) => {
+                    left.cmp(&right)
+                }
+                (LiteralValueTypeKind::Bool(_), _) => Ordering::Less,
+                (_, LiteralValueTypeKind::Bool(_)) => Ordering::Greater,
 
-        (Type::IntLiteral(left), Type::IntLiteral(right)) => left.cmp(right),
-        (Type::IntLiteral(_), _) => Ordering::Less,
-        (_, Type::IntLiteral(_)) => Ordering::Greater,
+                (LiteralValueTypeKind::Int(left), LiteralValueTypeKind::Int(right)) => {
+                    left.cmp(&right)
+                }
+                (LiteralValueTypeKind::Int(_), _) => Ordering::Less,
+                (_, LiteralValueTypeKind::Int(_)) => Ordering::Greater,
 
-        (Type::StringLiteral(left), Type::StringLiteral(right)) => left.cmp(right),
-        (Type::StringLiteral(_), _) => Ordering::Less,
-        (_, Type::StringLiteral(_)) => Ordering::Greater,
+                (LiteralValueTypeKind::String(left), LiteralValueTypeKind::String(right)) => {
+                    left.cmp(&right)
+                }
+                (LiteralValueTypeKind::String(_), _) => Ordering::Less,
+                (_, LiteralValueTypeKind::String(_)) => Ordering::Greater,
 
-        (Type::BytesLiteral(left), Type::BytesLiteral(right)) => left.cmp(right),
-        (Type::BytesLiteral(_), _) => Ordering::Less,
-        (_, Type::BytesLiteral(_)) => Ordering::Greater,
+                (LiteralValueTypeKind::Bytes(left), LiteralValueTypeKind::Bytes(right)) => {
+                    left.cmp(&right)
+                }
+                (LiteralValueTypeKind::Bytes(_), _) => Ordering::Less,
+                (_, LiteralValueTypeKind::Bytes(_)) => Ordering::Greater,
 
-        (Type::EnumLiteral(left), Type::EnumLiteral(right)) => left.cmp(right),
-        (Type::EnumLiteral(_), _) => Ordering::Less,
-        (_, Type::EnumLiteral(_)) => Ordering::Greater,
+                (LiteralValueTypeKind::Enum(left), LiteralValueTypeKind::Enum(right)) => {
+                    left.cmp(&right)
+                }
+            }
+        }
+
+        (Type::LiteralValue(_), _) => Ordering::Less,
+        (_, Type::LiteralValue(_)) => Ordering::Greater,
 
         (Type::FunctionLiteral(left), Type::FunctionLiteral(right)) => left.cmp(right),
         (Type::FunctionLiteral(_), _) => Ordering::Less,
@@ -117,6 +136,11 @@ pub(super) fn union_or_intersection_elements_ordering<'db>(
                 (SubclassOfInner::Dynamic(left), SubclassOfInner::Dynamic(right)) => {
                     dynamic_elements_ordering(left, right)
                 }
+                (SubclassOfInner::TypeVar(left), SubclassOfInner::TypeVar(right)) => {
+                    left.as_id().cmp(&right.as_id())
+                }
+                (SubclassOfInner::TypeVar(_), _) => Ordering::Less,
+                (_, SubclassOfInner::TypeVar(_)) => Ordering::Greater,
             }
         }
 
@@ -127,6 +151,10 @@ pub(super) fn union_or_intersection_elements_ordering<'db>(
         (Type::TypeIs(_), _) => Ordering::Less,
         (_, Type::TypeIs(_)) => Ordering::Greater,
 
+        (Type::TypeGuard(left), Type::TypeGuard(right)) => typeguard_ordering(db, *left, *right),
+        (Type::TypeGuard(_), _) => Ordering::Less,
+        (_, Type::TypeGuard(_)) => Ordering::Greater,
+
         (Type::NominalInstance(left), Type::NominalInstance(right)) => {
             left.class(db).cmp(&right.class(db))
         }
@@ -179,6 +207,17 @@ pub(super) fn union_or_intersection_elements_ordering<'db>(
                 }
                 (SuperOwnerKind::Instance(_), _) => Ordering::Less,
                 (_, SuperOwnerKind::Instance(_)) => Ordering::Greater,
+                (
+                    SuperOwnerKind::InstanceTypeVar(left, _),
+                    SuperOwnerKind::InstanceTypeVar(right, _),
+                ) => left.cmp(&right),
+                (SuperOwnerKind::InstanceTypeVar(..), _) => Ordering::Less,
+                (_, SuperOwnerKind::InstanceTypeVar(..)) => Ordering::Greater,
+                (SuperOwnerKind::ClassTypeVar(left, _), SuperOwnerKind::ClassTypeVar(right, _)) => {
+                    left.cmp(&right)
+                }
+                (SuperOwnerKind::ClassTypeVar(..), _) => Ordering::Less,
+                (_, SuperOwnerKind::ClassTypeVar(..)) => Ordering::Greater,
                 (SuperOwnerKind::Dynamic(left), SuperOwnerKind::Dynamic(right)) => {
                     dynamic_elements_ordering(left, right)
                 }
@@ -207,12 +246,14 @@ pub(super) fn union_or_intersection_elements_ordering<'db>(
         (Type::TypeAlias(_), _) => Ordering::Less,
         (_, Type::TypeAlias(_)) => Ordering::Greater,
 
-        (Type::TypedDict(left), Type::TypedDict(right)) => {
-            left.defining_class().cmp(&right.defining_class())
-        }
+        (Type::TypedDict(left), Type::TypedDict(right)) => left.cmp(right),
         (Type::TypedDict(_), _) => Ordering::Less,
         (_, Type::TypedDict(_)) => Ordering::Greater,
 
+        (Type::NewTypeInstance(left), Type::NewTypeInstance(right)) => left.cmp(right),
+        (Type::NewTypeInstance(_), _) => Ordering::Less,
+        (_, Type::NewTypeInstance(_)) => Ordering::Greater,
+
         (Type::Union(_), _) | (_, Type::Union(_)) => {
             unreachable!("our type representation does not permit nested unions");
         }
@@ -256,6 +297,12 @@ fn dynamic_elements_ordering(left: DynamicType, right: DynamicType) -> Ordering
         (DynamicType::Unknown, _) => Ordering::Less,
         (_, DynamicType::Unknown) => Ordering::Greater,
 
+        (DynamicType::UnknownGeneric(_), _) => Ordering::Less,
+        (_, DynamicType::UnknownGeneric(_)) => Ordering::Greater,
+
+        (DynamicType::UnspecializedTypeVar, _) => Ordering::Less,
+        (_, DynamicType::UnspecializedTypeVar) => Ordering::Greater,
+
         #[cfg(debug_assertions)]
         (DynamicType::Todo(TodoType(left)), DynamicType::Todo(TodoType(right))) => left.cmp(right),
 
@@ -265,24 +312,25 @@ fn dynamic_elements_ordering(left: DynamicType, right: DynamicType) -> Ordering
         (DynamicType::TodoUnpack, _) => Ordering::Less,
         (_, DynamicType::TodoUnpack) => Ordering::Greater,
 
-        (DynamicType::TodoTypeAlias, _) => Ordering::Less,
-        (_, DynamicType::TodoTypeAlias) => Ordering::Greater,
+        (DynamicType::TodoStarredExpression, _) => Ordering::Less,
+        (_, DynamicType::TodoStarredExpression) => Ordering::Greater,
 
-        (DynamicType::Divergent(left), DynamicType::Divergent(right)) => {
-            left.scope.cmp(&right.scope)
-        }
+        (DynamicType::TodoTypeVarTuple, _) => Ordering::Less,
+        (_, DynamicType::TodoTypeVarTuple) => Ordering::Greater,
+
+        (DynamicType::Divergent(left), DynamicType::Divergent(right)) => left.cmp(&right),
         (DynamicType::Divergent(_), _) => Ordering::Less,
         (_, DynamicType::Divergent(_)) => Ordering::Greater,
     }
 }
 
-/// Determine a canonical order for two instances of [`TypeIsType`].
+/// Generic helper for ordering type guard-like types.
 ///
 /// The following criteria are considered, in order:
 /// * Boundness: Unbound precedes bound
 /// * Symbol name: String comparison
 /// * Guarded type: [`union_or_intersection_elements_ordering`]
-fn typeis_ordering(db: &dyn Db, left: TypeIsType, right: TypeIsType) -> Ordering {
+fn guard_like_ordering<'db, T: TypeGuardLike<'db>>(db: &'db dyn Db, left: T, right: T) -> Ordering {
     let (left_ty, right_ty) = (left.return_type(db), right.return_type(db));
 
     match (left.place_info(db), right.place_info(db)) {
@@ -297,3 +345,13 @@ fn typeis_ordering(db: &dyn Db, left: TypeIsType, right: TypeIsType) -> Ordering
         },
     }
 }
+
+/// Determine a canonical order for two instances of [`TypeIsType`].
+fn typeis_ordering(db: &dyn Db, left: TypeIsType, right: TypeIsType) -> Ordering {
+    guard_like_ordering(db, left, right)
+}
+
+/// Determine a canonical order for two instances of [`TypeGuardType`].
+fn typeguard_ordering(db: &dyn Db, left: TypeGuardType, right: TypeGuardType) -> Ordering {
+    guard_like_ordering(db, left, right)
+}
