@@ -36,6 +36,7 @@ bitflags::bitflags! {
         const WORKSPACE_CONFIGURATION = 1 << 15;
         const COMPLETION_ITEM_LABEL_DETAILS_SUPPORT = 1 << 16;
         const DIAGNOSTIC_RELATED_INFORMATION = 1 << 17;
+        const PREFER_MARKDOWN_IN_COMPLETION = 1 << 18;
     }
 }
 
@@ -173,6 +174,11 @@ impl ResolvedClientCapabilities {
         self.contains(Self::COMPLETION_ITEM_LABEL_DETAILS_SUPPORT)
     }
 
+    /// Returns `true` if the client prefers Markdown over plain text in completion items.
+    pub(crate) const fn prefers_markdown_in_completion(self) -> bool {
+        self.contains(Self::PREFER_MARKDOWN_IN_COMPLETION)
+    }
+
     pub(super) fn new(client_capabilities: &ClientCapabilities) -> Self {
         let mut flags = Self::empty();
 
@@ -266,6 +272,24 @@ impl ResolvedClientCapabilities {
             flags |= Self::PREFER_MARKDOWN_IN_HOVER;
         }
 
+        if text_document
+            .and_then(|text_document| {
+                Some(
+                    text_document
+                        .completion
+                        .as_ref()?
+                        .completion_item
+                        .as_ref()?
+                        .documentation_format
+                        .as_ref()?
+                        .contains(&MarkupKind::Markdown),
+                )
+            })
+            .unwrap_or_default()
+        {
+            flags |= Self::PREFER_MARKDOWN_IN_COMPLETION;
+        }
+
         if text_document
             .and_then(|text_document| {
                 text_document
@@ -354,7 +378,7 @@ pub(crate) fn server_capabilities(
             // capabilities dynamically based on the `ty.diagnosticMode` setting.
             None
         } else {
-            // Otherwise, we always advertise support for workspace diagnostics.
+            // Otherwise, we always advertise support for workspace and pull diagnostics.
             Some(DiagnosticServerCapabilities::Options(
                 server_diagnostic_options(true),
             ))
@@ -423,6 +447,7 @@ pub(crate) fn server_capabilities(
             ..Default::default()
         }),
         selection_range_provider: Some(SelectionRangeProviderCapability::Simple(true)),
+        folding_range_provider: Some(types::FoldingRangeProviderCapability::Simple(true)),
         document_symbol_provider: Some(OneOf::Left(true)),
         workspace_symbol_provider: Some(OneOf::Left(true)),
         notebook_document_sync: Some(OneOf::Left(lsp_types::NotebookDocumentSyncOptions {
@@ -435,6 +460,15 @@ pub(crate) fn server_capabilities(
             }]
             .to_vec(),
         })),
+        workspace: Some(lsp_types::WorkspaceServerCapabilities {
+            workspace_folders: Some(lsp_types::WorkspaceFoldersServerCapabilities {
+                // N.B. It seems this is purely informational:
+                // https://github.com/microsoft/language-server-protocol/issues/1720#issuecomment-1514732305
+                supported: Some(true),
+                change_notifications: Some(OneOf::Left(true)),
+            }),
+            ..Default::default()
+        }),
         ..Default::default()
     }
 }
