@@ -97,6 +97,9 @@ pub(super) fn request(req: server::Request) -> Task {
         requests::SelectionRangeRequestHandler::METHOD => background_document_request_task::<
             requests::SelectionRangeRequestHandler,
         >(req, BackgroundSchedule::Worker),
+        requests::FoldingRangeRequestHandler::METHOD => background_document_request_task::<
+            requests::FoldingRangeRequestHandler,
+        >(req, BackgroundSchedule::Worker),
         requests::DocumentSymbolRequestHandler::METHOD => background_document_request_task::<
             requests::DocumentSymbolRequestHandler,
         >(req, BackgroundSchedule::Worker),
