@@ -1,13 +1,14 @@
 use std::collections::HashMap;
 
 use lsp_types::Url;
-use ruff_db::system::{SystemPath, SystemPathBuf};
+use ruff_db::system::{System, SystemPath, SystemPathBuf};
 use ruff_macros::Combine;
 use ruff_python_ast::PythonVersion;
 use serde::{Deserialize, Serialize};
 use serde_json::{Map, Value};
 use ty_combine::Combine;
 use ty_ide::{CompletionSettings, InlayHintSettings};
+use ty_project::CheckMode;
 use ty_project::metadata::Options as TyOptions;
 use ty_project::metadata::options::ProjectOptionsOverrides;
 use ty_project::metadata::value::{RangedValue, RelativePathBuf, ValueSource};
@@ -145,16 +146,16 @@ impl ClientOptions {
 #[serde(rename_all = "camelCase")]
 pub struct GlobalOptions {
     /// Diagnostic mode for the language server.
-    diagnostic_mode: Option<DiagnosticMode>,
+    pub diagnostic_mode: Option<DiagnosticMode>,
 
     /// Experimental features that the server provides on an opt-in basis.
-    pub(crate) experimental: Option<Experimental>,
+    pub experimental: Option<Experimental>,
 
     /// If `true` or [`None`], show syntax errors as diagnostics.
     ///
     /// This is useful when using ty with other language servers, allowing the user to refer
     /// to syntax errors from only one source.
-    pub(crate) show_syntax_errors: Option<bool>,
+    pub show_syntax_errors: Option<bool>,
 }
 
 impl GlobalOptions {
@@ -200,19 +201,32 @@ pub struct WorkspaceOptions {
 }
 
 impl WorkspaceOptions {
-    pub(crate) fn into_settings(self, root: &SystemPath, client: &Client) -> WorkspaceSettings {
-        let configuration_file =
-            self.configuration_file
-                .and_then(|config_file| match shellexpand::full(&config_file) {
-                    Ok(path) => Some(SystemPath::absolute(&*path, root)),
-                    Err(error) => {
-                        client.show_error_message(format_args!(
-                            "Failed to expand the environment variables \
-                            for the `ty.configuration_file` setting: {error}"
-                        ));
-                        None
-                    }
-                });
+    pub(crate) fn into_settings(
+        self,
+        root: &SystemPath,
+        client: &Client,
+        system: &dyn System,
+    ) -> WorkspaceSettings {
+        let configuration_file = self.configuration_file.and_then(|config_file| {
+            match shellexpand::full_with_context(
+                &config_file,
+                || system.env_var("HOME").ok(),
+                |var| match system.env_var(var) {
+                    Ok(val) => Ok(Some(val)),
+                    Err(std::env::VarError::NotPresent) => Ok(None),
+                    Err(e) => Err(e),
+                },
+            ) {
+                Ok(path) => Some(SystemPath::absolute(&*path, root)),
+                Err(error) => {
+                    client.show_error_message(format_args!(
+                        "Failed to expand the environment variables \
+                                for the `ty.configuration_file` setting: {error}"
+                    ));
+                    None
+                }
+            }
+        });
 
         let options_overrides =
             self.configuration.and_then(|map| {
@@ -369,6 +383,17 @@ impl DiagnosticMode {
         matches!(self, DiagnosticMode::OpenFilesOnly)
     }
 
+    /// Returns this diagnostic mode as a check mode.
+    ///
+    /// This returns `None` when diagnostics are disabled.
+    pub(crate) const fn to_check_mode(self) -> Option<CheckMode> {
+        match self {
+            DiagnosticMode::Off => None,
+            DiagnosticMode::OpenFilesOnly => Some(CheckMode::OpenFiles),
+            DiagnosticMode::Workspace => Some(CheckMode::AllFiles),
+        }
+    }
+
     pub(crate) const fn is_off(self) -> bool {
         matches!(self, DiagnosticMode::Off)
     }
@@ -396,7 +421,7 @@ impl Combine for DiagnosticMode {
     clippy::empty_structs_with_brackets,
     reason = "The LSP fails to deserialize the options when this is a unit type"
 )]
-pub(crate) struct Experimental {}
+pub struct Experimental {}
 
 impl Experimental {
     #[expect(clippy::unused_self)]
