@@ -27,6 +27,9 @@ import {
   type FileHandle,
   DocumentHighlight,
   DocumentHighlightKind,
+  InlayHintKind,
+  LocationLink,
+  TextEdit,
 } from "ty_wasm";
 import { FileId, ReadonlyFiles } from "../Playground";
 import { isPythonFile } from "./Files";
@@ -67,15 +70,25 @@ export default function Editor({
 }: Props) {
   const serverRef = useRef<PlaygroundServer | null>(null);
 
-  if (serverRef.current != null) {
-    serverRef.current.update({
-      files,
-      workspace,
-      onOpenFile,
-      onVendoredFileChange,
-      onBackToUserFile,
-    });
-  }
+  useEffect(() => {
+    serverRef.current?.update(
+      {
+        files,
+        workspace,
+        onOpenFile,
+        onVendoredFileChange,
+        onBackToUserFile,
+      },
+      isViewingVendoredFile,
+    );
+  }, [
+    files,
+    workspace,
+    onOpenFile,
+    onVendoredFileChange,
+    onBackToUserFile,
+    isViewingVendoredFile,
+  ]);
 
   // Update the diagnostics in the editor.
   useEffect(() => {
@@ -100,11 +113,7 @@ export default function Editor({
 
   useEffect(() => {
     return () => {
-      const server = serverRef.current;
-
-      if (server != null) {
-        server.dispose();
-      }
+      serverRef.current?.dispose();
     };
   }, []);
 
@@ -183,8 +192,11 @@ class PlaygroundServer
     languages.DocumentSemanticTokensProvider,
     languages.DocumentRangeSemanticTokensProvider,
     languages.SignatureHelpProvider,
-    languages.DocumentHighlightProvider
+    languages.DocumentHighlightProvider,
+    languages.CodeActionProvider
 {
+  private diagnostics: Diagnostic[] = [];
+
   private typeDefinitionProviderDisposable: IDisposable;
   private declarationProviderDisposable: IDisposable;
   private definitionProviderDisposable: IDisposable;
@@ -198,6 +210,8 @@ class PlaygroundServer
   private rangeSemanticTokensDisposable: IDisposable;
   private signatureHelpDisposable: IDisposable;
   private documentHighlightDisposable: IDisposable;
+  private codeActionDisposable: IDisposable;
+  private inVendoredFileCondition: editor.IContextKey<boolean>;
   // Cache for vendored file handles
   private vendoredFileHandles = new Map<string, FileHandle>();
 
@@ -246,9 +260,21 @@ class PlaygroundServer
       monaco.languages.registerSignatureHelpProvider("python", this);
     this.documentHighlightDisposable =
       monaco.languages.registerDocumentHighlightProvider("python", this);
+    this.codeActionDisposable = monaco.languages.registerCodeActionProvider(
+      "python",
+      this,
+    );
 
+    this.inVendoredFileCondition = editor.createContextKey<boolean>(
+      "inVendoredFile",
+      false,
+    );
     // Register Esc key command
-    editor.addCommand(monaco.KeyCode.Escape, this.props.onBackToUserFile);
+    editor.addCommand(
+      monaco.KeyCode.Escape,
+      () => this.props.onBackToUserFile(),
+      "inVendoredFile",
+    );
   }
 
   triggerCharacters: string[] = ["."];
@@ -311,15 +337,31 @@ class PlaygroundServer
     const digitsLength = String(completions.length - 1).length;
 
     return {
+      incomplete: true,
       suggestions: completions.map((completion, i) => ({
-        label: completion.name,
+        label: {
+          label: completion.name,
+          detail:
+            completion.module_name == null ||
+            completion.additional_text_edits == null
+              ? undefined
+              : ` (import ${completion.module_name})`,
+          description: completion.detail ?? undefined,
+        },
         sortText: String(i).padStart(digitsLength, "0"),
         kind:
           completion.kind == null
             ? CompletionItemKind.Variable
             : mapCompletionKind(completion.kind),
-        insertText: completion.name,
+        insertText: completion.insert_text ?? completion.name,
+        additionalTextEdits: completion.additional_text_edits?.map(
+          (edit: TextEdit) => ({
+            range: tyRangeToMonacoRange(edit.range),
+            text: edit.new_text,
+          }),
+        ),
         documentation: completion.documentation,
+        detail: completion.detail,
         // TODO(micha): It's unclear why this field is required for monaco but not VS Code.
         //  and omitting it works just fine? The LSP doesn't expose this information right now
         //  which is why we go with undefined for now.
@@ -405,14 +447,43 @@ class PlaygroundServer
       return undefined;
     }
 
+    function mapInlayHintKind(kind: InlayHintKind): languages.InlayHintKind {
+      switch (kind) {
+        case InlayHintKind.Type:
+          return languages.InlayHintKind.Type;
+        case InlayHintKind.Parameter:
+          return languages.InlayHintKind.Parameter;
+      }
+    }
+
     return {
       dispose: () => {},
       hints: inlayHints.map((hint) => ({
-        label: hint.markdown,
+        label: hint.label.map((part) => {
+          const locationLink = part.location
+            ? this.mapNavigationTarget(part.location)
+            : undefined;
+
+          return {
+            label: part.label,
+            // Range cannot be `undefined`.
+            location: locationLink?.targetSelectionRange
+              ? {
+                  uri: locationLink.uri,
+                  range: locationLink.targetSelectionRange,
+                }
+              : undefined,
+          };
+        }),
         position: {
           lineNumber: hint.position.line,
           column: hint.position.column,
         },
+        kind: mapInlayHintKind(hint.kind),
+        textEdits: hint.text_edits.map((edit: TextEdit) => ({
+          range: tyRangeToMonacoRange(edit.range),
+          text: edit.new_text,
+        })),
       })),
     };
   }
@@ -426,8 +497,9 @@ class PlaygroundServer
     return undefined;
   }
 
-  update(props: PlaygroundServerProps) {
+  update(props: PlaygroundServerProps, isViewingVendoredFile: boolean) {
     this.props = props;
+    this.inVendoredFileCondition.set(isViewingVendoredFile);
   }
 
   private getOrCreateVendoredFileHandle(vendoredPath: string): FileHandle {
@@ -491,6 +563,8 @@ class PlaygroundServer
   }
 
   updateDiagnostics(diagnostics: Array<Diagnostic>) {
+    this.diagnostics = diagnostics;
+
     if (this.props.files.selected == null) {
       return;
     }
@@ -541,6 +615,68 @@ class PlaygroundServer
     );
   }
 
+  provideCodeActions(
+    model: editor.ITextModel,
+    range: Range,
+    // eslint-disable-next-line @typescript-eslint/no-unused-vars
+    _context: languages.CodeActionContext,
+    // eslint-disable-next-line @typescript-eslint/no-unused-vars
+    _token: CancellationToken,
+  ): languages.ProviderResult<languages.CodeActionList> {
+    const actions: languages.CodeAction[] = [];
+    const fileHandle = this.getFileHandleForModel(model);
+    if (fileHandle == null) {
+      return undefined;
+    }
+
+    for (const diagnostic of this.diagnostics) {
+      const diagnosticRange = diagnostic.range;
+      if (diagnosticRange == null) {
+        continue;
+      }
+
+      const monacoRange = tyRangeToMonacoRange(diagnosticRange);
+      if (!Range.areIntersecting(range, monacoRange)) {
+        continue;
+      }
+
+      const codeActions = this.props.workspace.codeActions(
+        fileHandle,
+        diagnostic.raw,
+      );
+      if (codeActions == null) {
+        continue;
+      }
+
+      for (const codeAction of codeActions) {
+        actions.push({
+          title: codeAction.title,
+          kind: "quickfix",
+          isPreferred: codeAction.preferred,
+          edit: {
+            edits: codeAction.edits.map((edit) => ({
+              resource: model.uri,
+              textEdit: {
+                range: tyRangeToMonacoRange(edit.range),
+                text: edit.new_text,
+              },
+              versionId: model.getVersionId(),
+            })),
+          },
+        });
+      }
+    }
+
+    if (actions.length === 0) {
+      return undefined;
+    }
+
+    return {
+      actions,
+      dispose: () => {},
+    };
+  }
+
   provideHover(
     model: editor.ITextModel,
     position: Position,
@@ -724,57 +860,59 @@ class PlaygroundServer
     return null;
   }
 
-  private mapNavigationTargets(links: any[]): languages.LocationLink[] {
-    const result = links.map((link) => {
-      const uri = Uri.parse(link.path);
+  private mapNavigationTarget(link: LocationLink): languages.LocationLink {
+    const uri = Uri.parse(link.path);
 
-      // Pre-create models to ensure peek definition works
-      if (this.monaco.editor.getModel(uri) == null) {
-        if (uri.scheme === "vendored") {
-          // Handle vendored files
-          const vendoredPath = this.getVendoredPath(uri);
-          const fileHandle = this.getOrCreateVendoredFileHandle(vendoredPath);
-          const content = this.props.workspace.sourceText(fileHandle);
-          this.monaco.editor.createModel(content, "python", uri);
-        } else {
-          // Handle regular files
-          const fileId = this.props.files.index.find((file) => {
-            return Uri.file(file.name).toString() === uri.toString();
-          })?.id;
+    // Pre-create models to ensure peek definition works
+    if (this.monaco.editor.getModel(uri) == null) {
+      if (uri.scheme === "vendored") {
+        // Handle vendored files
+        const vendoredPath = this.getVendoredPath(uri);
+        const fileHandle = this.getOrCreateVendoredFileHandle(vendoredPath);
+        const content = this.props.workspace.sourceText(fileHandle);
+        this.monaco.editor.createModel(content, "python", uri);
+      } else {
+        // Handle regular files
+        const fileId = this.props.files.index.find((file) => {
+          return Uri.file(file.name).toString() === uri.toString();
+        })?.id;
 
-          if (fileId != null) {
-            const handle = this.props.files.handles[fileId];
-            if (handle != null) {
-              const language = isPythonFile(handle) ? "python" : undefined;
-              this.monaco.editor.createModel(
-                this.props.files.contents[fileId],
-                language,
-                uri,
-              );
-            }
+        if (fileId != null) {
+          const handle = this.props.files.handles[fileId];
+          if (handle != null) {
+            const language = isPythonFile(handle) ? "python" : undefined;
+            this.monaco.editor.createModel(
+              this.props.files.contents[fileId],
+              language,
+              uri,
+            );
           }
         }
       }
+    }
 
-      const targetSelection =
-        link.selection_range == null
-          ? undefined
-          : tyRangeToMonacoRange(link.selection_range);
+    const targetSelection =
+      link.selection_range == null
+        ? undefined
+        : tyRangeToMonacoRange(link.selection_range);
 
-      const originSelection =
-        link.origin_selection_range == null
-          ? undefined
-          : tyRangeToMonacoRange(link.origin_selection_range);
+    const originSelection =
+      link.origin_selection_range == null
+        ? undefined
+        : tyRangeToMonacoRange(link.origin_selection_range);
 
-      return {
-        uri: uri,
-        range: tyRangeToMonacoRange(link.full_range),
-        targetSelectionRange: targetSelection,
-        originSelectionRange: originSelection,
-      } as languages.LocationLink;
-    });
+    return {
+      uri: uri,
+      range: tyRangeToMonacoRange(link.full_range),
+      targetSelectionRange: targetSelection,
+      originSelectionRange: originSelection,
+    } as languages.LocationLink;
+  }
 
-    return result;
+  private mapNavigationTargets(
+    links: LocationLink[],
+  ): languages.LocationLink[] {
+    return links.map((link) => this.mapNavigationTarget(link));
   }
 
   dispose() {
@@ -791,6 +929,7 @@ class PlaygroundServer
     this.completionDisposable.dispose();
     this.signatureHelpDisposable.dispose();
     this.documentHighlightDisposable.dispose();
+    this.codeActionDisposable.dispose();
   }
 }
 
